<?
/*
TribesQ v0.02.12.09
Tribes 2 Query
Written by Janrex (janrex@hotpop.com)
http://psoft.port5.com

Redirections:
http://psoft.tk
http://psoft.da.ru
http://psoft.tsx.org

Purpose:
  Create a simple PHP class for querying Tribes 2 servers.

Notes:
  You can do whatever you want with this. If you make any modifications, have any ideas
  or bug reports then please email me at janrex@email.com or go to the website at
  http://psoft.port5.com. If you actually use this then please notify me so I know it's
  actually useful. I will also put a link to your site if you wish.

Requirements:
  PHP4 (does not work with PHP3)
  PHP4 enabled server with access through firewall (firewall may block the server queries)

Tested Web Servers:
  Apache/1.3.12 (Unix) (Red Hat/Linux) PHP/4.0.1pl2
  Apache/1.3.12 (Unix) (Red Hat/Linux) PHP/4.0.4pl1
  Apache/1.3.14 (Win32) PHP/4.0.4pl1
  Apache/1.3.14 (Win32) PHP/4.0.5
  Apache/1.3.14 (Unix)  PHP/4.0.4
  Apache/1.3.20 (Win32) PHP/4.1.1
  Apache/1.3.27 (Win32) PHP/4.2.3

Bugs:
  -ping estimation code may not properly on a windows system.

Usage:
  -look at tribesqexample.php.

History:
  -Query packet string fixed
  -SaveRawToFile now returns successful or not
  -added support for query VER5
  -added support for statusbyte bit for linux
  -added $TribesQ->Player->TribeName and $TribesQ->Player->TribeNameType
  -players that are part of a tribe should be properly flagged with registered now
  -changed $TribesQ->Server->TeamScore to $TribesQ->Server->TeamScoreTotal (total score of players in team)
  -changed $TribesQ->Server->TeamScoreText to $TribesQ->Server->TeamScore (flag caps)
  -the dns reverse lookup can get a dns record but the dns may not actually point to the real ip
  -$TribesQ->Player->TribesNameType == (0 = no tribename, 1 = prepend tribename, 2 = append tribename)
  -problem with servers with a lot of people because udp packet size is limited 1500 bytes
  -$TribesQ->Server->MinNetProtocol and $TribesQ->Server->NetProtocol
*/

// player record holding specific player information
class RPlayer {
  var $Name,$Score,$TeamName,$TribeName,$TribeNameType;
  //var $Ping,$PL,$Team;
  var $FlagsByte,$Flags;
}

// server record holding specific server information
class RServer {
  var $Name,$Type,$ModName,$MissionType,$Map,$Info,$GameString,$GameVersion; // strings
  var $Dedicated,$ResponseType,$Password,$NumPlayers,$MaxPlayers,$NumBots,$NumTeams,$Flags,$StatusByte; // numbers
  var $TeamName,$TeamScore,$TeamScoreTotal,$Status; // arrays
  var $ID,$CPU,$QID1,$QID2,$QueryVersion,$Ping,$Content,$NoGame,$NetProtocol,$MinNetProtocol; // numbers
}

class TTribesQ {
  var $sRaw;   // raw response stream
  var $iLast;  // last position
  var $sHost;  // host to query
  var $iPort;  // host's port to query
  var $sResolvedHost; // hostip resolved to hostname
  var $fp;     // filepointer for response
  var $Server; // holds server info
  var $Player; // holds player info
  var $ErrNo, $ErrStr; // filled if there are any errors in connection attempt
  var $DebugInfo; // holds debug information

  // constructor
  function TTribesQ()
  {
    $this->Server = new RServer;
    $this->Player = array(new RPlayer);
    $iLast = 0;
    $this->ErrNo = -1;
    set_magic_quotes_runtime(0); // needed for reading buffer properly
    define(QUERY_TRIBES1_MASTER,"\x10\x03\xFF\x00\x02");
    define(QUERY_TRIBES1_INFO,"\x62\x01\x01");
    define(QUERY_TRIBES2_MASTER,"\x06\x00\x00\x00\x00\x00\xFF\x03\x61\x6E\x79\x03\x61\x6E\x79\x00\xFF\xFF\xFF\xFF\xFF\x00");
    define(QUERY_TRIBES2_PING,"\x0E\x02\x01\x02\x03\x04");
    define(QUERY_TRIBES2_INFO,"\x12\x02\x01\x02\x03\x04");
    //define(QUERY_TRIBES2_PING,"\x0E\x02\x01\x01\x01\x01");
    //define(QUERY_TRIBES2_INFO,"\x12\x02\x01\x01\x01\x01");
  }

  // destructor - not automatic, must be manually called
  function Destroy()
  {
    unset($this->Server);
    unset($this->Player);
  }

  // finish the query
  function Finish()
  {
    set_magic_quotes_runtime(get_magic_quotes_gpc());
    if ($this->fp)
    {
      //socket_set_blocking($this->fp,0);
      fclose($this->fp);
    }
  }

  // returns unix time including microseconds
  function utime()
  {
    $t = microtime();
    $t = ((double)strstr($t,' ')+(double)substr($t,0,strpos($t,' ')));
    return (double)$t;
  }
  // read a single byte from socket
  function ReadByte()
  {
    $this->sRaw .= fread($this->fp,1);
    $this->iLast++;
    return Ord($this->sRaw[$this->iLast-1]);
  }

  // read a string from socket where first byte is string length (pascal string)
  function ReadString()
  {
    $iLen = $this->ReadByte();
    $this->sRaw .= fread($this->fp,$iLen);
    for ($i=0;$i<$iLen;$i++) {
      $s .= $this->sRaw[$this->iLast+$i];
    }
    $this->iLast += $iLen;
    return $s;
  }

  // read a number terminated by a specified byte
  function ReadTermInteger($terminator = 0x0A)
  {
    do {
      $tmp = $this->sRaw[$this->iLast++];
      if (Ord($tmp) == $terminator || Ord($tmp) == 0x11) break;
      if ($this->iLast > strlen($this->sRaw)) break;
      $s .= $tmp;
    } while (Ord($tmp) != $terminator || Ord($tmp) != 0x11);
    return $s;
  }

  // read a string terminated by a specified byte
  function ReadTermString($terminator = 0x09)
  {
    return ($this->ReadTermInteger($terminator));
  }

  // begin query, two optional parameters: resolve the host and connect timeout
  // returns true if successful and false if failed
  function Query($resolvehost = false,$timeout = 3)
  {
    if ((int)$resolvehost) {
      if (ereg ("([0-9]{1,3})[.]([0-9]{1,3})[.]([0-9]{1,3})[.]([0-9]{1,3})", $this->sHost, $reg))
        $this->sResolvedHost = gethostbyaddr($this->sHost);
      else
        $this->sResolvedHost = $this->sHost;
    }
    $this->fp = fsockopen("udp://$this->sHost",$this->iPort,&$this->ErrNo,&$this->ErrStr,$timeout);
    if (!$this->fp) {
      $this->Finish();
      return false;
    } else {
      $iTickCount = $this->utime();
      fwrite($this->fp,QUERY_TRIBES2_PING);
      if ($this->ReadByte() == 0x10) {
        $this->Server->Ping = round($this->utime()*1000 - $iTickCount*1000,0);
        $this->ParseResponse1();
        unset($this->sRaw);
        $this->iLast = 0;
        $iTickCount = $this->utime();
        fwrite($this->fp,QUERY_TRIBES2_INFO);
        if ($this->ReadByte() == 0x14) {
          $this->Server->Ping = Round(($this->Server->Ping + round($this->utime()*1000 - $iTickCount*1000,0))/2);
          $this->ParseResponse2();
        }
        $this->Finish();
        return true;
      }
      $this->Finish();
      return false;
    }
  }

  function ParseResponse1()
  {
    $this->Server->Flags = $this->ReadByte();
    $this->Server->QID2 = $this->ReadByte().$this->ReadByte().$this->ReadByte().$this->ReadByte();
    $this->Server->QueryVersion = $this->ReadString();
    if ($this->Server->QueryVersion == "VER3") {
      $this->Server->MinNetProtocol = $this->ReadByte();
      for ($i=0;$i<3;$i++)
        $this->ReadByte();
    } else {
      $this->Server->NetProtocol = $this->ReadByte();
      for ($i=0;$i<3;$i++)
        $this->ReadByte();
      $this->Server->MinNetProtocol = $this->ReadByte();
      for ($i=0;$i<3;$i++)
        $this->ReadByte();
    }
    $max = $this->ReadByte();
    $min = $this->ReadByte();
    $this->Server->BuildVersion = HexDec(DecHex($min).DecHex($max));
    $this->ReadByte();
    $this->ReadByte();
    $this->Server->Name = $this->ReadString();
  }

  function ParseResponse2()
  {
    $this->Server->Flags = $this->ReadByte();
    $this->Server->QID1 = $this->ReadByte().$this->ReadByte().$this->ReadByte().$this->ReadByte();
    $this->Server->ModName = $this->ReadString();
    $this->Server->MissionType = $this->ReadString();
    $this->Server->Map = $this->ReadString();
    $this->Server->StatusByte = $this->ReadByte();
    $this->Server->Status["stDedicated"] = ($this->Server->StatusByte & 1);
    $this->Server->Status["stPassword"] = ($this->Server->StatusByte & 2)/2;
    $this->Server->Status["stLinux"] = ($this->Server->StatusByte & 4)/4;
    if ($this->Server->QueryVersion == "VER3") {
      $this->Server->Status["stTournament"] = ($this->Server->StatusByte & 64)/64;
      $this->Server->Status["stNoAlias"] = ($this->Server->StatusByte & 128)/128;
    } else {
      $this->Server->Status["stTournament"] = ($this->Server->StatusByte & 8)/8;
      $this->Server->Status["stNoAlias"] = ($this->Server->StatusByte & 16)/16;
    }
    $this->Server->NumPlayers = $this->ReadByte();
    $this->Server->MaxPlayers = $this->ReadByte();
    $this->Server->NumBots = $this->ReadByte();
    $this->Server->CPU = $this->ReadByte()+$this->ReadByte()*256;
    $this->Server->Info = $this->ReadString();
    $this->Server->Content = $this->ReadByte()+$this->ReadByte()*256;
    $this->sRaw .= fread($this->fp,$this->Server->Content);
    $this->Server->NoGame = (int)($this->Server->Content == 6);
    if (!$this->Server->NoGame)
    {
      $this->Server->NumTeams = $this->ReadTermInteger();
      for ($i=0;$i<$this->Server->NumTeams;$i++) {
        $this->Server->TeamName[] = $this->ReadTermString();
        $this->Server->TeamScore[] = $this->ReadTermInteger();
      }
      $tmp = $this->ReadTermInteger();
      for ($i=0;$i<$tmp;$i++) {
        $this->iLast++;
        $this->Player[$i]->FlagsByte = Ord($this->sRaw[$this->iLast++]);
        $this->Player[$i]->Flags["pfRegistered"] = ($this->Player[$i]->FlagsByte == 8) || ($this->Player[$i]->FlagsByte == 0x0B);
        $this->Player[$i]->Flags["pfAlias"] = ($this->Player[$i]->FlagsByte == 12);
        $this->Player[$i]->Flags["pfBot"] = ($this->Player[$i]->FlagsByte == 14);
        $this->Player[$i]->Name = $this->ReadTermString();
        if (strrpos($this->Player[$i]->Name,0x08)) { // tribename prepended
          $name = substr($this->Player[$i]->Name,strrpos($this->Player[$i]->Name,0x08)+1,strlen($this->Player[$i]->Name)-strrpos($this->Player[$i]->Name,0x08));
          $tribe = substr($this->Player[$i]->Name,0,strrpos($this->Player[$i]->Name,0x08));
          $this->Player[$i]->Name = $name;
          $this->Player[$i]->TribeName = $tribe;
          $this->Player[$i]->TribeNameType = 1;
        } else if (strrpos($this->Player[$i]->Name,0x0B) && strrpos($this->Player[$i]->Name,0x0B) != strlen($this->Player[$i]->Name)-1) { // tribename appended
          $name = substr($this->Player[$i]->Name,0,strrpos($this->Player[$i]->Name,0x0B));
          $tribe = substr($this->Player[$i]->Name,strrpos($this->Player[$i]->Name,0x0B)+1,strlen($this->Player[$i]->Name)-strrpos($this->Player[$i]->Name,0x0B));
          $this->Player[$i]->Name = $name;
          $this->Player[$i]->TribeName = $tribe;
          $this->Player[$i]->TribeNameType = 2;
        } else { // no tribename
          if (strrpos($this->Player[$i]->Name,0x0B))
            $this->Player[$i]->Name = substr($this->Player[$i]->Name,0,strrpos($this->Player[$i]->Name,0x0B));
          $this->Player[$i]->TribeNameType = 0;
        }
        $this->ReadTermString();
        $this->Player[$i]->TeamName = $this->ReadTermString();
        if (!$this->Player[$i]->TeamName)
          $this->Player[$i]->TeamName = "-";
        $this->Player[$i]->Score = $this->ReadTermInteger();
        for ($x=0;$x<$this->Server->NumTeams;$x++)
          if ($this->Server->TeamName[$x] == $this->Player[$i]->TeamName)
            $this->Server->TeamScoreTotal[$x] += $this->Player[$i]->Score;
      }
    }
  }

  // save raw response text to file, file must exist and chmod to 666
  function SaveRawToFile($filename)
  {
    if (is_writeable($filename))
    {
      $this->fp = fopen($filename,"w");
      fwrite($this->fp,$this->sRaw,strlen($this->sRaw));
      fclose($this->fp);
      return true;
    } else
      return false;
  }

}
// convert space to "&nbsp;", less than to "&lt;", etc.
function Convert2HTML($s)
{
  return ereg_replace(" ","&nbsp;",htmlentities($s));
}
?>
